% Demo script reproducing the experiments reported in
% L. Azzari and A. Foi, "Variance Stabilization in Poisson Image Deblurring", Proc. IEEE ISBI 2017
%
%  v1.00, April 18, 2017
%
% http://www.cs.tut.fi/~foi/invansc/
%
%  L. Azzari and A. Foi - Tampere University of Technology - 2017 - All rights reserved.
% -----------------------------------------------------------------------------------------------
%
%   !! IMPORTANT !!
%   This demo requires:
%    * Exact unbiased inverse package "invansc"
%       http://www.cs.tut.fi/~foi/invansc/
%    * BM3D denoising filter
%       http://www.cs.tut.fi/~foi/GCF-BM3D/
% -----------------------------------------------------------------------------------------------


%% load image to restore -- the image should be in the range [0 1]
img_name='fluocells.tif';
% img_name='cameraman.tif';
% img_name='moon_gray.png';
y=im2double(imread(['images_ISBI',filesep,img_name]));

%% choose image peak value
peak = 255;
y=y/max(y(:))*peak;

%% blur point-spread function (PSF) and blurring
v=fspecial('gaussian', 25, sqrt(3));
g = imfilter(y, v(end:-1:1,end:-1:1), 'circular'); 

%% seed for noise realization
ii=1; randn('seed',ii);  rand('seed',ii);
z=poissrnd(g);

%% Deblur via iterative (VST)+(colored noise removal)
% yhat_RWI is the final estimate
% yhat_RI is the estimate after the first stage (standard regularized inverse)
[yhat_RWI,yhat_RI] = iterVSTpoissonDeb(z,v);

%%  show also intermediate RI result, or only RWI
showRI=0;

%% compute PSNR of the estimate
PSNR_z=10*log10(peak^2/mean((y(:)-z(:)).^2));
PSNR_yhat_RI=10*log10(peak^2/mean((y(:)-yhat_RI(:)).^2));
PSNR_yhat_RWI=10*log10(peak^2/mean((y(:)-yhat_RWI(:)).^2));
resString=['PSNR(z): '     num2str(PSNR_z,'%4.2f'),'dB'];
if showRI
    resString=[resString, '   PSNR(yhat_RI): '  num2str(PSNR_yhat_RI,'%4.2f'),'dB'];
end
resString=[resString, '   PSNR(yhat_RWI): ' num2str(PSNR_yhat_RWI,'%4.2f'),'dB'];
disp(resString);


%% visualize results
figure
subplot(1,2+showRI,1)
imshow(z,[0 peak])
title(['Noisy blurred $z$ (PSNR: ' num2str(PSNR_z,'%4.2f') ' dB)'],'Interpreter','latex','fontsize',16)
if showRI
    subplot(1,2+showRI,2)
    imshow(yhat_RI,[0 peak])
    title(['Restored $\hat{y}$ (PSNR: ' num2str(PSNR_yhat_RI,'%4.2f') ' dB)'],'Interpreter','latex','fontsize',16)
end
subplot(1,2+showRI,2+showRI)
imshow(yhat_RWI,[0 peak])
title(['Restored $\hat{y}$ (PSNR: ' num2str(PSNR_yhat_RWI,'%4.2f') ' dB)'],'Interpreter','latex','fontsize',16)
truesize
